"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var STANDARD_JSON_SETTINGS_KEYS = [
    "stopAfter", "remappings", "optimizer", "evmVersion", "debug", "metadata", "libraries", "outputSelection"
];
/**
 * Abstraction of a checked solidity contract. With metadata and source (solidity) files.
 * The info property contains the information about compilation or errors encountered while validating the metadata.
 */
var CheckedContract = /** @class */ (function () {
    function CheckedContract(metadata, solidity, missing, invalid) {
        this.metadata = JSON.parse(JSON.stringify(metadata));
        this.solidity = this.sourceMapToStringMap(solidity);
        this.missing = missing;
        this.invalid = invalid;
        var sources = this.metadata.sources;
        for (var compiledPath in sources) {
            var metadataSource = sources[compiledPath];
            var foundSource = solidity[compiledPath];
            if (!metadataSource.content && foundSource) {
                metadataSource.content = foundSource.content;
            }
            delete metadataSource.license;
        }
        var _a = this.getPathAndName(), contractPath = _a.contractPath, contractName = _a.contractName;
        this.compiledPath = contractPath;
        this.name = contractName;
        this.info = this.isValid() ? this.composeSuccessMessage() : this.composeErrorMessage();
    }
    /** Checks whether this contract is valid or not.
     * @returns true if no sources are missing or are invalid (malformed); false otherwise
     */
    CheckedContract.prototype.isValid = function () {
        return isEmpty(this.missing) && isEmpty(this.invalid);
    };
    CheckedContract.prototype.sourceMapToStringMap = function (input) {
        var ret = {};
        for (var key in input) {
            ret[key] = input[key].content;
        }
        return ret;
    };
    /**
     * Returns the contract path and name as specified in the metadata provided during construction.
     */
    CheckedContract.prototype.getPathAndName = function () {
        var compilationTarget = this.metadata.settings.compilationTarget;
        var contractPath = Object.keys(compilationTarget)[0];
        var contractName = compilationTarget[contractPath];
        return { contractPath: contractPath, contractName: contractName };
    };
    /**
     * Returns a JSON object compatible with solc --standard-json.
     *
     * @param useOriginalSettings If true, the returned object will contain settings as specified
     * during the original compilation; otherwise no settings will be imposed.
     */
    CheckedContract.prototype.getStandardJson = function (useOriginalSettings) {
        if (useOriginalSettings === void 0) { useOriginalSettings = true; }
        var standardJson = {
            language: "Solidity",
            sources: this.metadata.sources
        };
        if (useOriginalSettings && this.metadata.settings) {
            standardJson.settings = {};
            standardJson.settings.outputSelection = { "*": { "*": ["evm.bytecode", "abi"] } };
            for (var _i = 0, STANDARD_JSON_SETTINGS_KEYS_1 = STANDARD_JSON_SETTINGS_KEYS; _i < STANDARD_JSON_SETTINGS_KEYS_1.length; _i++) {
                var key = STANDARD_JSON_SETTINGS_KEYS_1[_i];
                if (this.metadata.settings.hasOwnProperty(key)) {
                    standardJson.settings[key] = this.metadata.settings[key];
                }
            }
        }
        return standardJson;
    };
    /**
     * Constructs the message to be displayed in case of a successful finding
     * of source files related to the provided metadata file.
     */
    CheckedContract.prototype.composeSuccessMessage = function () {
        var compilerVersionDetailed = this.metadata.compiler.version;
        var compilerVersion = compilerVersionDetailed.split("+")[0];
        var msgLines = [];
        msgLines.push(this.name + " (" + this.compiledPath + "):");
        msgLines.push("  Success!");
        msgLines.push("  Compiled with Solidity " + compilerVersion);
        msgLines.push("  https://solc-bin.ethereum.org/wasm/soljson-v" + compilerVersionDetailed + ".js");
        msgLines.push("  https://solc-bin.ethereum.org/linux-amd64/solc-linux-amd64-v" + compilerVersionDetailed);
        return msgLines.join("\n");
    };
    /**
     * Constructs the message to be displayed in case the contract is not valid.
     * The structure of the message is as follows:
     *
     * path: name
     *
     *   missing sources
     *
     *   invalid sources
     *
     *   number of found sources
     */
    CheckedContract.prototype.composeErrorMessage = function () {
        var msgLines = [];
        msgLines.push(this.name + " (" + this.compiledPath + "):");
        if (!isEmpty(this.missing)) {
            msgLines.push("  Error: Missing sources:");
            msgLines.push("  The following files were not provided (or were altered, so their hash doesn't match the one in the metadata).");
            msgLines.push("  Please retrieve the files (potentially via ipfs) and re-run the script.");
        }
        for (var missingSourceName in this.missing) {
            msgLines.push("    " + missingSourceName + ":");
            var missingSourceProps = this.missing[missingSourceName];
            for (var prop in missingSourceProps) {
                var propValue = missingSourceProps[prop];
                if (Array.isArray(propValue)) {
                    propValue.forEach(function (elem) {
                        msgLines.push("      " + elem);
                    });
                }
                else {
                    msgLines.push("      " + prop + ": " + propValue);
                }
            }
        }
        if (!isEmpty(this.invalid)) {
            msgLines.push("  Error: Invalid sources:");
        }
        for (var invalidSourceName in this.invalid) {
            msgLines.push("    " + invalidSourceName + ":");
            msgLines.push("      " + this.invalid[invalidSourceName]);
        }
        var foundSourcesNumber = Object.keys(this.solidity).length;
        if (foundSourcesNumber) {
            msgLines.push("  " + foundSourcesNumber + " other source files found successfully.");
        }
        return msgLines.join("\n");
    };
    return CheckedContract;
}());
exports.default = CheckedContract;
/**
 * Checks whether the provided object contains any keys or not.
 * @param obj The object whose emptiness is tested.
 * @returns true if any keys present; false otherwise
 */
function isEmpty(obj) {
    return !Object.keys(obj).length;
}
//# sourceMappingURL=CheckedContract.js.map