/// <reference types="node" />
import bunyan from 'bunyan';
import CheckedContract from './CheckedContract';
export declare class PathBuffer {
    path?: string;
    buffer: Buffer;
    constructor(buffer: Buffer, path?: string);
}
declare class PathContent {
    path: string;
    content: string;
    constructor(content: string, path?: string);
}
export interface SourceMap {
    [compiledPath: string]: PathContent;
}
export interface IValidationService {
    /**
     * Checks all metadata files found in the provided paths. Paths may include regular files, directoris and zip archives.
     *
     * @param paths The array of paths to be searched and checked.
     * @param ignoring Optional array where all unreadable paths can be stored.
     * @returns An array of CheckedContract objects.
     * @throws Error if no metadata files are found.
     */
    checkPaths(paths: string[], ignoring?: string[]): CheckedContract[];
    /**
     * Checks the files provided in the array of Buffers. May include buffers of zip archives.
     * Attempts to find all the resources specified in every metadata file found.
     *
     * @param files The array of buffers to be checked.
     * @returns An array of CheckedContract objets.
     * @throws Error if no metadata files are found.
     */
    checkFiles(files: PathBuffer[]): CheckedContract[];
}
export declare class ValidationService implements IValidationService {
    logger: bunyan;
    /**
     * @param logger a custom logger that logs all errors; undefined or no logger provided turns the logging off
     */
    constructor(logger?: bunyan);
    checkPaths(paths: string[], ignoring?: string[]): CheckedContract[];
    checkFiles(files: PathBuffer[]): CheckedContract[];
    /**
     * Traverses the given files, unzipping any zip archive.
     *
     * @param files the array containing the files to be checked
     * @returns an array containing the provided files, with any zips being unzipped and returned
     */
    private findInputFiles;
    /**
     * Checks whether the provided file is in fact zipped.
     * @param file the buffered file to be checked
     * @returns true if the file is zipped; false otherwise
     */
    private isZip;
    /**
     * Unzips the provided file buffer to the provided array.
     *
     * @param zippedFile the buffer containin the zipped file to be unpacked
     * @param files the array to be filled with the content of the zip
     */
    private unzip;
    private sanitizeInputFiles;
    /**
     * Selects metadata files from an array of files that may include sources, etc
     * @param  {string[]} files
     * @return {string[]}         metadata
     */
    private findMetadataFiles;
    /**
     * Validates metadata content keccak hashes for all files and
     * returns mapping of file contents by file name
     * @param  {any}       metadata
     * @param  {string[]}  files    source files
     * @return foundSources, missingSources, invalidSources
     */
    private rearrangeSources;
    /**
     * Generates a map of files indexed by the keccak hash of their content.
     *
     * @param  {string[]}  files Array containing sources.
     * @returns Map object that maps hash to PathContent.
     */
    private storeByHash;
    private extractMetadataFromString;
    /**
     * A method that checks if the provided object was generated as a metadata file of a Solidity contract.
     * Current implementation is rather simplistic and may require further engineering.
     *
     * @param metadata the JSON to be checked
     * @returns true if the provided object is a Solidity metadata file; false otherwise
     */
    private isMetadata;
    /**
     * Applies the provided worker function to the provided path recursively.
     *
     * @param path the path to be traversed
     * @param worker the function to be applied on each file that is not a directory
     * @param afterDir the function to be applied on the directory after traversing its children
     */
    private traversePathRecursively;
}
export {};
